package com.huimv.eartag.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.admin.entity.AccountGroup;
import com.huimv.admin.entity.SysAccountMultilevel;
import com.huimv.admin.mapper.AccountGroupMapper;
import com.huimv.admin.mapper.SysAccountMultilevelMapper;
import com.huimv.common.utils.ChineseCharToEnUtil;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.eartag.entity.BaseFarmInfo;
import com.huimv.eartag.entity.BaseFarmInfo;
import com.huimv.eartag.mapper.BaseFarmInfoMapper;
import com.huimv.eartag.service.IBaseFarmInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author astupidcoder
 * @since 2022-01-07
 */
@Service
public class BaseFarmInfoServiceImpl extends ServiceImpl<BaseFarmInfoMapper, BaseFarmInfo> implements IBaseFarmInfoService {
    @Autowired
    private SysAccountMultilevelMapper accountMultilevelMapper;

    @Autowired
    private AccountGroupMapper accountGroupMapper;

    @Autowired
    private BaseFarmInfoMapper baseFarmInfoMapper;

    @Override
    public Result pageAll(Map map) {
        Integer current = (Integer)map.get("current");
        Integer size = (Integer) map.get("size");
        String farmName = (String) map.get("farmName");
        String location = (String)map.get("location");
        Integer type = (Integer)map.get("type");
        QueryWrapper<BaseFarmInfo> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(farmName),"farm_name",farmName);
        wrapper.like(StringUtils.isNotBlank(location),"location",location);
        if (type != null){
            wrapper.eq("type",type);
        }
        Page<BaseFarmInfo> page = page(new Page<>(current, size), wrapper);
        return new Result(ResultCode.SUCCESS,page);
    }

    @Override
    @Transactional
    public Result saveFarmAndAccount(BaseFarmInfo baseFarmInfo) {

        //创建人
        SysAccountMultilevel accountMultilevel = accountMultilevelMapper.selectById(baseFarmInfo.getCreateUser());

        SysAccountMultilevel account = new SysAccountMultilevel();
        account.setPassword("123456");

        //设置账号
        String farmName = baseFarmInfo.getFarmName();

        String firstSpell = ChineseCharToEnUtil.getFirstSpell(farmName);
        List<SysAccountMultilevel> sysAccountMultilevels = accountMultilevelMapper.selectList(new QueryWrapper<SysAccountMultilevel>().like("account", firstSpell));
        if (ObjectUtil.isEmpty(sysAccountMultilevels)){
            account.setAccount(firstSpell+"01");
        }else {
            Integer number = 0;
            for (SysAccountMultilevel sysAccountMultilevel : sysAccountMultilevels) {
                String oldName = sysAccountMultilevel.getAccount();
                int oldNumber = Integer.parseInt(oldName.substring(oldName.length() - 2));
                if (oldNumber>number){
                    number = oldNumber;
                }
            }
            number++;
           if (number < 10){
               account.setAccount(firstSpell+"0"+number);
           }else if ( number<100){
               account.setAccount(firstSpell+number);
           }else {
               return new Result(10001,"该牧场账号已达到上限",false);
           }
        }

        account.setPid(accountMultilevel.getId());
        account.setPids(accountMultilevel.getPids()+","+accountMultilevel.getId());
        account.setUserName(baseFarmInfo.getFarmName());
        account.setType(baseFarmInfo.getType());

        baseFarmInfo.setFirstAccount(account.getAccount());
        int insert = baseFarmInfoMapper.insert(baseFarmInfo);
        if (insert> 0){
            account.setFarmIds(baseFarmInfo.getId()+"");
        }

        accountMultilevelMapper.insert(account);
        Integer accountId = account.getId();
        AccountGroup accountGroup = new AccountGroup();
        accountGroup.setAccountId(accountId);
        //设置权限
        if (baseFarmInfo.getType() == 0){

            accountGroup.setGroupId(31);
        }
        //屠宰
        else if (baseFarmInfo.getType() == 1){
            accountGroup.setGroupId(34);
        }
        //养殖
        else if (baseFarmInfo.getType() == 2){
            accountGroup.setGroupId(32);
        }
        //检疫
        else if (baseFarmInfo.getType() == 3){
            accountGroup.setGroupId(33);
        }
        return new Result(10000,"添加成功",true);
    }

    @Override
    @Transactional
    public Result removeByIdsAndAccount(List<Integer> ids) {
        List<BaseFarmInfo> baseFarmInfos = this.listByIds(ids);
        for (BaseFarmInfo baseFarmInfo : baseFarmInfos) {

            accountMultilevelMapper.delete(new QueryWrapper<SysAccountMultilevel>().eq("account", baseFarmInfo.getFirstAccount()));
        }
        this.removeByIds(ids);
        return new Result(10000,"删除成功",true);
    }
}
