package com.huimv.eartag.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.eartag.entity.*;
import com.huimv.eartag.mapper.*;
import com.huimv.eartag.service.EartagRegisterService;
import com.huimv.eartag.service.IBizInspectionPhotoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.eartag.utils.UpImage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author astupidcoder
 * @since 2022-03-03
 */
@Service
public class BizInspectionPhotoServiceImpl extends ServiceImpl<BizInspectionPhotoMapper, BizInspectionPhoto> implements IBizInspectionPhotoService {

   private static final String PICURL = "/opt/goldpig-huimv/img";



    @Autowired
    private BizFirstInspecEartagMapper bizFirstInspecEartagMapper;

    @Autowired
    private BizSecondInspecEartagMapper bizSecondInspecEartagMapper;

    @Autowired
    private BizSellPigApplyMapper bizSellPigApplyMapper;

    @Autowired
    private EartagRegisterMapper eartagRegisterMapper;

    @Autowired
    private BizSellPigApplyEartagMapper bizSellPigApplyEartagMapper;

    @Override
    @Transactional
    public Result addInspection(HttpServletRequest request, MultipartFile[] files, String farmId,String batchNo,Integer inspecStatus,String inspecRremark,Integer inspecRealNum) {


        List<BizSellPigApplyEartag> bizSellPigApplyEartags = bizSellPigApplyEartagMapper.selectList(new QueryWrapper<BizSellPigApplyEartag>().eq("batch_no", batchNo));
        List<String> eartagNos = bizSellPigApplyEartags.stream().map(BizSellPigApplyEartag::getEarmark).collect(Collectors.toList());

        DateTime date = DateUtil.date();



        //检疫明细
        for (String eartag : eartagNos) {
            BizFirstInspecEartag bizFirstInspecEartag = new BizFirstInspecEartag();
            bizFirstInspecEartag.setBatchNo(batchNo);
            bizFirstInspecEartag.setCreateTime(date);
            bizFirstInspecEartag.setEarmark(eartag);
            bizFirstInspecEartag.setFarmId(farmId);
            bizFirstInspecEartagMapper.insert(bizFirstInspecEartag);
        }

        //修改卖猪记录状态
        BizSellPigApply bizSellPigApply = bizSellPigApplyMapper.selectOne(new QueryWrapper<BizSellPigApply>().eq("batch_no", batchNo));
        bizSellPigApply.setInspecFlowStatus(1);
        bizSellPigApply.setInspecRemark(inspecRremark);
        bizSellPigApply.setInspecRealNum(inspecRealNum);

        bizSellPigApply.setInspecTime(date);
        if (inspecStatus == 0){
            bizSellPigApply.setRecordStatus(4);
        }
        bizSellPigApplyMapper.updateById(bizSellPigApply);

        //修改耳标注册记录状态
        EartagRegisterEntity eartagRegisterEntity =new EartagRegisterEntity();
        eartagRegisterEntity.setStage(12);
        UpdateWrapper<EartagRegisterEntity> updateWrapper =new UpdateWrapper<>();
        updateWrapper.in("eartag_no", eartagNos);
        eartagRegisterMapper.update(eartagRegisterEntity,updateWrapper);
        if (ObjectUtil.isNotEmpty(files)) {

            //图片存储
            List<BizInspectionPhoto> bizInspectionPhotos = new ArrayList<>();
            String s = UpImage.uploadImgs(files, PICURL, request);
            String[] imgUrls = s.split(",");
            for (String imgUrl : imgUrls) {
                BizInspectionPhoto bizInspectionPhoto = new BizInspectionPhoto();
                bizInspectionPhoto.setPhotoName(imgUrl.substring(imgUrl.lastIndexOf("/")));
                bizInspectionPhoto.setBatchNo(batchNo);
                bizInspectionPhoto.setPhotoPath(imgUrl);
                bizInspectionPhoto.setPhotoType(1);

                bizInspectionPhotos.add(bizInspectionPhoto);
            }
            this.saveBatch(bizInspectionPhotos);
        }

        return new Result(ResultCode.SUCCESS,"上传成功");
    }

    @Override
    public Result addBatch(HttpServletRequest request, MultipartFile[] files, String farmId,String batchNo,Integer inspecStatus,String inspecRremark,Integer inspecRealNum) {
        List<BizFirstInspecEartag> bizSellPigApplyEartags = bizFirstInspecEartagMapper.selectList(new QueryWrapper<BizFirstInspecEartag>().eq("batch_no", batchNo));
        List<String> eartagNos = bizSellPigApplyEartags.stream().map(BizFirstInspecEartag::getEarmark).collect(Collectors.toList());

        DateTime date = DateUtil.date();



        //检疫明细
        for (String eartag : eartagNos) {
            BizSecondInspecEartag bizSecondInspecEartag = new BizSecondInspecEartag();
            bizSecondInspecEartag.setBatchNo(batchNo);
            bizSecondInspecEartag.setCreateTime(date);
            bizSecondInspecEartag.setEarmark(eartag);
            bizSecondInspecEartag.setFarmId(farmId);
            bizSecondInspecEartagMapper.insert(bizSecondInspecEartag);
        }

        //修改卖猪记录状态
        BizSellPigApply bizSellPigApply = bizSellPigApplyMapper.selectOne(new QueryWrapper<BizSellPigApply>().eq("batch_no", batchNo));
        bizSellPigApply.setInspecFlowStatus(2);
        bizSellPigApply.setButcherRemark(inspecRremark);
        bizSellPigApply.setButcherRealNum(inspecRealNum);
        bizSellPigApply.setRecordStatus(3);
        bizSellPigApply.setButcherTime(date);
        if (inspecStatus == 0){
            bizSellPigApply.setRecordStatus(5);
        }
        bizSellPigApplyMapper.updateById(bizSellPigApply);

        //修改耳标注册记录状态
        EartagRegisterEntity eartagRegisterEntity =new EartagRegisterEntity();
        eartagRegisterEntity.setStage(13);
        UpdateWrapper<EartagRegisterEntity> updateWrapper =new UpdateWrapper<>();
        updateWrapper.in("eartag_no", eartagNos);
        eartagRegisterMapper.update(eartagRegisterEntity,updateWrapper);

        if (ObjectUtil.isNotEmpty(files)) {

            //图片存储
            List<BizInspectionPhoto> bizInspectionPhotos = new ArrayList<>();
            String s = UpImage.uploadImgs(files, PICURL, request);
            String[] imgUrls = s.split(",");
            for (String imgUrl : imgUrls) {
                BizInspectionPhoto bizInspectionPhoto = new BizInspectionPhoto();
                bizInspectionPhoto.setPhotoName(imgUrl.substring(imgUrl.lastIndexOf("/")));
                bizInspectionPhoto.setBatchNo(batchNo);
                bizInspectionPhoto.setPhotoPath(imgUrl);
                bizInspectionPhoto.setPhotoType(2);

                bizInspectionPhotos.add(bizInspectionPhoto);
            }
            this.saveBatch(bizInspectionPhotos);
        }

        return new Result(ResultCode.SUCCESS,"上传成功");
    }

    @Override
    public Result listPic(Map<String,String> map) {
        String batchNo = map.get("batchNo");
        String types = map.get("types");
        String[] split = types.split(",");
        List<BizInspectionPhoto> list = this.list(new QueryWrapper<BizInspectionPhoto>().eq("batch_no", batchNo).in("photo_type", split));
        return new Result(ResultCode.SUCCESS,list);
    }
}
