package com.huimv.eartag.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.eartag.entity.BizFirstInspecEartag;
import com.huimv.eartag.entity.BizSellPigApply;
import com.huimv.eartag.entity.BizSellPigApplyEartag;
import com.huimv.eartag.entity.EartagRegisterEntity;
import com.huimv.eartag.entity.vo.BizSellPigApplyVo;
import com.huimv.eartag.entity.vo.EartagNoVo;
import com.huimv.eartag.entity.vo.InspecVo;
import com.huimv.eartag.mapper.BizFirstInspecEartagMapper;
import com.huimv.eartag.mapper.BizSellPigApplyEartagMapper;
import com.huimv.eartag.mapper.BizSellPigApplyMapper;
import com.huimv.eartag.mapper.EartagRegisterMapper;
import com.huimv.eartag.service.EartagRegisterService;
import com.huimv.eartag.service.IBizSellPigApplyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author astupidcoder
 * @since 2022-03-01
 */
@Service
public class BizSellPigApplyServiceImpl extends ServiceImpl<BizSellPigApplyMapper, BizSellPigApply> implements IBizSellPigApplyService {

    @Autowired
    private BizSellPigApplyMapper bizSellPigApplyMapper;

    @Autowired
    private BizSellPigApplyEartagMapper bizSellPigApplyEartagMapper;

    @Autowired
    private EartagRegisterMapper eartagRegisterMapper;

    @Autowired
    private BizFirstInspecEartagMapper bizFirstInspecEartagMapper;

    @Override
    public Result sellPigRecord(Map<String,String>  map) {
        String farmId = map.get("farmId");
        String type = map.get("type");

        String current = map.get("current");
        String size =  map.get("size");

        Page<BizSellPigApply> bizSellPigApplyPage ;
        if (StringUtils.isBlank(current) || StringUtils.isBlank(size)){
            bizSellPigApplyPage = new Page<>(1, 10);
        }else {
            bizSellPigApplyPage =new Page<>(Integer.parseInt(current), Integer.parseInt(size));
        }
        QueryWrapper<BizSellPigApply> wrapper = new QueryWrapper<>();
        wrapper.eq("farm_id",farmId);
        wrapper.orderByDesc("sell_time");
        //未提交
        if ("1".equals(type)){
            wrapper.eq("record_status",0);
        }
        //进行中
        else if("2".equals(type)){
            wrapper.eq("record_status",1);
        }
        //已完成
        else if ("3".equals(type)){
            wrapper.eq("record_status",3);
        }

        //失败
        else if("4".equals(type)){
            wrapper.in("record_status",4,5);
        }

        //全部

        Page<BizSellPigApply> page = this.page(bizSellPigApplyPage, wrapper);
        return new Result(ResultCode.SUCCESS,page);
    }

    @Override
    public Result sellPigRecordCollect(Integer farmId) {

        BizSellPigApplyVo bizSellPigApplyVo = new BizSellPigApplyVo();

        QueryWrapper<BizSellPigApply> wrapper = new QueryWrapper<>();
        wrapper.eq("farm_id",farmId);
        bizSellPigApplyVo.setAllPigNum(this.count(wrapper));

        wrapper.eq("record_status",3);
        bizSellPigApplyVo.setCompletedNum(this.count(wrapper));

        wrapper.clear();
        wrapper.eq("farm_id",farmId);
        wrapper.in("record_status",4,5);
        bizSellPigApplyVo.setFailNum(this.count(wrapper));


        wrapper.clear();
        wrapper.eq("farm_id",farmId);
        wrapper.eq("record_status",1);
        bizSellPigApplyVo.setUnderwayNum(this.count(wrapper));

        wrapper.clear();
        wrapper.eq("farm_id",farmId);
        wrapper.eq("record_status",2);
        bizSellPigApplyVo.setUncommitted(this.count(wrapper));

        return new Result(ResultCode.SUCCESS,bizSellPigApplyVo);
    }

    //很low的卖猪流程，别看
    @Override
    public Result sellPigRecordManage(String batchNo) {


        Map endMap = new HashMap();
        endMap.put("now",0);
        List list = new ArrayList();

        Map map1 = new HashMap();
        map1.put("title","提交申请");
        map1.put("subTitle","未提交");

        Map map2 = new HashMap();
        map2.put("title","检疫部门检疫");
        map2.put("subTitle","检疫部门未检疫");

        Map map3 = new HashMap();
        map3.put("title","屠宰部门检疫");
        map3.put("subTitle","屠宰部门未检疫");

        Map map4 = new HashMap();
        map4.put("title","已屠宰");
        map4.put("subTitle","未屠宰");




        BizSellPigApply batch = this.getOne(new QueryWrapper<BizSellPigApply>().eq("batch_no", batchNo));

        String inspecTime = DateUtil.format(batch.getInspecTime(), "yyyy-MM-dd HH:mm:ss");
        String sellTime = DateUtil.format(batch.getSellTime(), "yyyy-MM-dd HH:mm:ss");
        String butcherInspecTime = DateUtil.format(batch.getButcherInspecTime(), "yyyy-MM-dd HH:mm:ss");
        String butcherTime = DateUtil.format(batch.getButcherTime(), "yyyy-MM-dd HH:mm:ss");

        //0:暂存 1：已提交 2：取消 3:已完成 4:检疫失败 5:屠宰失败
        Integer recordStatus = batch.getRecordStatus();

        //检验流程状态 0：未检验， 1:在场检验，2：屠宰检验
        Integer inspecFlowStatus = batch.getInspecFlowStatus();
        if (1 == recordStatus ){
            map1.put("subTitle",sellTime);
            if (1 == inspecFlowStatus){
                map2.put("subTitle",inspecTime);
                endMap.put("now",1);
            }
            if ( 2== inspecFlowStatus){
                map2.put("subTitle",inspecTime);
                map3.put("subTitle",butcherInspecTime);
                endMap.put("now",2);
            }
        }
        if ( 3== recordStatus){
            map1.put("subTitle",sellTime);
            map2.put("subTitle",inspecTime);
            map3.put("subTitle",butcherInspecTime);
            map4.put("subTitle",butcherTime);
            endMap.put("now",3);
        }
        if ( 4==recordStatus){
            map1.put("subTitle",sellTime);
            map2.put("subTitle","检疫失败");
            endMap.put("now",1);
        }

        if ( 5==recordStatus){
            map1.put("subTitle",sellTime);
            map2.put("subTitle",inspecTime);
            map3.put("subTitle","屠宰检疫失败");
            endMap.put("now",2);
        }
        list.add(map1);
        list.add(map2);
        list.add(map3);
        list.add(map4);
        endMap.put("data",list);
        endMap.put("inspecPic",null);
        endMap.put("butcherPic",null);


        return new Result(ResultCode.SUCCESS,endMap);
    }

    @Override
    public Result getInspecCollect(String farmId) {
        InspecVo inspecVo = new InspecVo();
        QueryWrapper<BizSellPigApply> wrapper = new QueryWrapper<>();
        wrapper.eq("inspec_org_id",farmId);
        inspecVo.setInspecNum(this.count(wrapper));


        wrapper.clear();
        wrapper.eq("inspec_org_id",farmId);
        wrapper.eq("inspec_flow_status",0);
        int count = this.count(wrapper);
        inspecVo.setAccomplishNum(count);



        wrapper.clear();
        wrapper.eq("inspec_org_id",farmId);
        wrapper.in("inspec_flow_status",1,2);
        int count2 = this.count(wrapper);


        wrapper.eq("record_status",4);
        int count1 = this.count(wrapper);
        inspecVo.setUnQualifiedNum(count1);


        inspecVo.setQualifiedNum(count2 - count1);

        return new Result(ResultCode.SUCCESS,inspecVo);
    }

    @Override
    public Result getInspecRecord(Map<String,String> map) {
        String farmId = map.get("farmId");
        String type = map.get("type");

        String current = map.get("current");
        String size =  map.get("size");
        Page<BizSellPigApply> bizSellPigApplyPage ;
        if (StringUtils.isBlank(current) || StringUtils.isBlank(size)){
            bizSellPigApplyPage = new Page<>(1, 10);
        }else {
            bizSellPigApplyPage =new Page<>(Integer.parseInt(current), Integer.parseInt(size));
        }


        QueryWrapper<BizSellPigApply> wrapper = new QueryWrapper<>();
        wrapper.eq("inspec_org_id",farmId);
        wrapper.orderByDesc("sell_time");
        //未审批
        if ("1".equals(type)){
            wrapper.eq("inspec_flow_status",0);

        }
        //合格
       else if ("2".equals(type)){
            wrapper.in("inspec_flow_status",1,2);
            wrapper.ne("record_status",4);
        }

        //不合格
       else if ("3".equals(type)){
            wrapper.in("inspec_flow_status",1,2);
            wrapper.eq("record_status",4);
        }
       //全部


        Page<BizSellPigApply> page = this.page(bizSellPigApplyPage, wrapper);
        return new Result(ResultCode.SUCCESS,page);
    }

    @Override
    public Result inspecCheck(Map<String,String> map) {

        Map endMap = new HashMap();

        String batchNo = map.get("batchNo");
        String farmId = map.get("farmId");
        List<BizSellPigApplyEartag> inspecNos = bizSellPigApplyEartagMapper.selectList(new QueryWrapper<BizSellPigApplyEartag>().eq("batch_no", batchNo));
        // DOTO
        List<EartagRegisterEntity> eartagRegisterEntities = eartagRegisterMapper.selectList(new QueryWrapper<EartagRegisterEntity>().eq("stage", 11).eq("inspec_id", farmId));

        List<String> inspecNos1 = inspecNos.stream().map(BizSellPigApplyEartag::getEarmark).collect(Collectors.toList());
        List<String> eartagRegister1 = eartagRegisterEntities.stream().map(EartagRegisterEntity::getEartagNo).collect(Collectors.toList());
        int inspecNos1Size = inspecNos1.size();

        List<EartagNoVo> list = new ArrayList<>();
        inspecNos1.stream().forEach( s -> list.add(new EartagNoVo(s,0)));
        endMap.put("total",inspecNos1Size);

       inspecNos1.removeAll(eartagRegister1);

        endMap.put("eartagNos",list);
        BizSellPigApply bizSellPigApply = this.getOne(new QueryWrapper<BizSellPigApply>().eq("batch_no", batchNo));

        endMap.put("realEartagNum",bizSellPigApply.getRealEartagNum());
        endMap.put("autoEartagNum",bizSellPigApply.getAutoEartagNum());
        if (inspecNos1.size() == 0){
            for (String s : inspecNos1) {
                list.stream().filter(o -> o.getEartagNo().equals(s)).forEach(eartagNoVo ->  eartagNoVo.setStatus(1));
            }
            endMap.put("inspecAutoNum",inspecNos1Size);
        }else {
            inspecNos1Size =  inspecNos1Size - inspecNos1.size();
            endMap.put("inspecAutoNum", inspecNos1Size);

        }


        bizSellPigApply.setInspecAutoNum(inspecNos1Size);

        this.updateById(bizSellPigApply);
        return new Result(ResultCode.SUCCESS,endMap);
    }



    //屠宰检疫
    @Override
    public Result getBatchCollect(String farmId) {
        InspecVo inspecVo = new InspecVo();
        QueryWrapper<BizSellPigApply> wrapper = new QueryWrapper<>();
        wrapper.eq("butcher_org_id",farmId);
        wrapper.in("inspec_flow_status",1,2);
        inspecVo.setInspecNum(this.count(wrapper));


        wrapper.clear();
        wrapper.eq("butcher_org_id",farmId);
        wrapper.eq("inspec_flow_status",1);
        int count = this.count(wrapper);
        inspecVo.setAccomplishNum(count);


        wrapper.clear();
        wrapper.eq("butcher_org_id",farmId);
        wrapper.eq("inspec_flow_status",2);
        int count2 = this.count(wrapper);

        wrapper.eq("record_status",5);
        int count1 = this.count(wrapper);
        inspecVo.setUnQualifiedNum(count1);


        inspecVo.setQualifiedNum(count2 -count1);

        return new Result(ResultCode.SUCCESS,inspecVo);
    }

    @Override
    public Result getBatchRecord(Map<String, String> map) {
        String farmId = map.get("farmId");
        String type = map.get("type");

        String current = map.get("current");
        String size =  map.get("size");
        Page<BizSellPigApply> bizSellPigApplyPage ;
        if (StringUtils.isBlank(current) || StringUtils.isBlank(size)){
            bizSellPigApplyPage = new Page<>(1, 10);
        }else {
            bizSellPigApplyPage =new Page<>(Integer.parseInt(current), Integer.parseInt(size));
        }


        QueryWrapper<BizSellPigApply> wrapper = new QueryWrapper<>();
        wrapper.eq("butcher_org_id",farmId);
        wrapper.orderByDesc("sell_time");
        wrapper.ne("record_status",4);
        //未审批
        if ("1".equals(type)){
            wrapper.eq("inspec_flow_status",1);

        }
        //合格
        else if ("2".equals(type)){
            wrapper.eq("inspec_flow_status",2);
            wrapper.ne("record_status",5);
        }

        //不合格
        else if ("3".equals(type)){
            wrapper.eq("inspec_flow_status",2);
            wrapper.eq("record_status",5);
        }
        //全部
        else {
            wrapper.in("inspec_flow_status",1,2);
        }



        Page<BizSellPigApply> page = this.page(bizSellPigApplyPage, wrapper);
        return new Result(ResultCode.SUCCESS,page);
    }

    @Override
    public Result batchCheck(Map<String, String> map) {
        Map endMap = new HashMap();

        String batchNo = map.get("batchNo");
        String farmId = map.get("farmId");
        List<BizFirstInspecEartag> inspecNos = bizFirstInspecEartagMapper.selectList(new QueryWrapper<BizFirstInspecEartag>().eq("batch_no", batchNo));
        // DOTO
        List<EartagRegisterEntity> eartagRegisterEntities = eartagRegisterMapper.selectList(new QueryWrapper<EartagRegisterEntity>().eq("stage", 12).eq("butcher_id", farmId));

        List<String> inspecNos1 = inspecNos.stream().map(BizFirstInspecEartag::getEarmark).collect(Collectors.toList());
        List<String> eartagRegister1 = eartagRegisterEntities.stream().map(EartagRegisterEntity::getEartagNo).collect(Collectors.toList());
        int inspecNos1Size = inspecNos1.size();

        List<EartagNoVo> list = new ArrayList<>();
        inspecNos1.stream().forEach( s -> list.add(new EartagNoVo(s,0)));
        endMap.put("total",inspecNos1Size);

        inspecNos1.removeAll(eartagRegister1);

        endMap.put("eartagNos",list);
        BizSellPigApply bizSellPigApply = this.getOne(new QueryWrapper<BizSellPigApply>().eq("batch_no", batchNo));

        endMap.put("realEartagNum",bizSellPigApply.getInspecRealNum());
        endMap.put("autoEartagNum",bizSellPigApply.getInspecAutoNum());


        if (inspecNos1.size() == 0){
            endMap.put("inspecAutoNum",inspecNos1Size);
        }else {
            for (String s : inspecNos1) {
                list.stream().filter(o -> o.getEartagNo().equals(s)).forEach(eartagNoVo ->  eartagNoVo.setStatus(1));
            }
            inspecNos1Size =  inspecNos1Size - inspecNos1.size();
            endMap.put("inspecAutoNum", inspecNos1Size);

        }


        bizSellPigApply.setButcherAutoNum(inspecNos1Size);

        this.updateById(bizSellPigApply);
        return new Result(ResultCode.SUCCESS,endMap);
    }
}
