package com.huimv.eartag.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.common.utils.PageFactory;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.eartag.entity.BasePigpen;
import com.huimv.eartag.entity.CollectorRegister;
import com.huimv.eartag.entity.dto.CollectorRegisterParam;
import com.huimv.eartag.mapper.CollectorRegisterMapper;
import com.huimv.eartag.service.ICollectorRegisterService;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author astupidcoder
 * @since 2022-01-07
 */
@Service
public class CollectorRegisterServiceImpl extends ServiceImpl<CollectorRegisterMapper, CollectorRegister> implements ICollectorRegisterService {


    @Override
    public Result pageAll(Map map) {
        Integer current = (Integer)map.get("current");
        Integer size = (Integer) map.get("size");
        String searchName = (String) map.get("searchName");
        String date =(String)map.get("registerTime");
        String pigpenId =(String)map.get("pigpenId");
        String stage =(String)map.get("stage");

        QueryWrapper<CollectorRegister> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(searchName),"device_code",searchName);
        wrapper.eq("pigpen_id",pigpenId);
        wrapper.eq("stage",stage);
        //TODO 注册时间有BUG
        wrapper.eq("register_time",date);


        Page<CollectorRegister> page = page(new Page<>(current, size), wrapper);

        return new Result(ResultCode.SUCCESS,page);
    }


    @Override
    public Result page(CollectorRegisterParam collectorRegisterParam) {

        QueryWrapper<CollectorRegister> queryWrapper = new QueryWrapper<>();
            if (ObjectUtil.isNotEmpty(collectorRegisterParam.getRegisterTime())) {
                Date registerTime = collectorRegisterParam.getRegisterTime();
                DateTime startTime = DateUtil.beginOfDay(registerTime);
                DateTime endTime = DateUtil.endOfDay(registerTime);
                queryWrapper.lambda().ge(CollectorRegister::getRegisterTime,startTime);
                queryWrapper.lambda().le(CollectorRegister::getRegisterTime,endTime);
            }

            // 根据设备编码查询
            if (ObjectUtil.isNotEmpty(collectorRegisterParam.getPigpenId())) {
                queryWrapper.lambda().like(CollectorRegister::getPigpenId, collectorRegisterParam.getPigpenId());
            }

            // 根据设备编码查询
            if (ObjectUtil.isNotEmpty(collectorRegisterParam.getDeviceCode())) {
                queryWrapper.lambda().like(CollectorRegister::getDeviceCode, collectorRegisterParam.getDeviceCode());
            }
            // 根据状态
            if (ObjectUtil.isNotEmpty(collectorRegisterParam.getCanStatus())) {
                queryWrapper.lambda().like(CollectorRegister::getCanStatus, collectorRegisterParam.getCanStatus());
            }
             //根据牧场
            if (ObjectUtil.isNotEmpty(collectorRegisterParam.getFarmId())) {
             queryWrapper.lambda().eq( CollectorRegister::getFarmCode,  String.valueOf(collectorRegisterParam.getFarmId()) );
         }
            
            
            
            
            

              Page<CollectorRegister> page = page(new Page<>(collectorRegisterParam.getPageNo(), collectorRegisterParam.getPageSize()), queryWrapper);

            return new Result(ResultCode.SUCCESS,page);
    }



    @Override
    public Result listCount(Integer farmId) {

        LambdaQueryWrapper<CollectorRegister> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.in(CollectorRegister::getFarmCode, farmId);

        Map<String, Integer> map = new LinkedHashMap<>();
        map.put("total", count(lambdaQuery));

        lambdaQuery.eq(CollectorRegister::getCanStatus, 0);
        map.put("normal", count(lambdaQuery));

        lambdaQuery.clear();
        lambdaQuery.eq(CollectorRegister::getFarmCode, farmId);
        lambdaQuery.ne(CollectorRegister::getCanStatus, 0);
        map.put("abnormal", count(lambdaQuery));

        lambdaQuery.clear();
        lambdaQuery.eq(CollectorRegister::getFarmCode, farmId);
        lambdaQuery.eq(CollectorRegister::getCanStatus, 3);
        map.put("logout", count(lambdaQuery));

        return  new Result(ResultCode.SUCCESS,map);

    }
}
